#+ELEVENTY_COLLECTIONS: _posts
#+ELEVENTY_BASE_DIR: ~/proj/static-blog/

* Using bug-hunter to quickly find a problem in my long Emacs configuration  :emacs:
:PROPERTIES:
:EXPORT_DATE: 2023-01-14T10:06:32-0500
:EXPORT_ELEVENTY_PERMALINK: /blog/2023/01/using-bug-hunter-to-quickly-find-a-problem-in-my-long-emacs-configuration/
:EXPORT_ELEVENTY_FILE_NAME: blog/2023/01/using-bug-hunter-to-quickly-find-a-problem-in-my-long-emacs-configuration/
:END:

I noticed that author names in my mastodon.el timeline were no longer
getting highlighted. Since it behaved correctly when I used ~emacs
-Q~, I knew the problem was somewhere in my configuration. Problem: my
configuration is very long. (Uh, my .el seems to have grown to be
about 460KB...) I started writing some code to help me bisect it, but
fortunately I remembered using a package to do that a long time ago.
After a little digging around, I found [[https://github.com/Malabarba/elisp-bug-hunter][bug-hunter]] again. I [[https://github.com/Malabarba/elisp-bug-hunter/pull/36][patched it]]
to allow me to specify a header and footer:

#+begin_src emacs-lisp :eval no
(defvar bug-hunter-header nil "Emacs Lisp sexp to add at the beginning of the file.")
(defvar bug-hunter-footer nil "Emacs Lisp sexp to add at the end of the file.")

(defun bug-hunter--print-to-temp (sexp)
  "Print SEXP to a temp file and return the file name."
  (let ((print-length nil)
        (print-level nil)
        (file (make-temp-file "bug-hunter")))
    (with-temp-file file
      (when bug-hunter-header (print bug-hunter-header (current-buffer)))
      (print sexp (current-buffer))
      (when bug-hunter-footer (print bug-hunter-footer (current-buffer))))
    file))
#+end_src

That allowed me to set up this header and footer:

#+begin_src emacs-lisp
(setq bug-hunter-header
			'(progn
				 (package-initialize)
				 (use-package quelpa)
				 (use-package quelpa-use-package)
				 (use-package hydra :commands defhydra)
				 (use-package use-package-hydra)))
(setq bug-hunter-footer '(load-file "/tmp/mastodon-test.el"))
#+end_src

where mastodon-test.el was this file:

[[my-include:/tmp/mastodon-test.el::wrap=src emacs-lisp]]

I wasn't sure how to use bug-hunter's support for assertions because
of the asynchronous retrieval of the Mastodon timeline, but at least I
could use that code to set things up.

Then I used ~bug-hunter-file~ to split up the Emacs Lisp file tangled
out of my literate configuration. All I had to do was wait for the
Mastodon timeline to show up, quit Emacs, and answer ~y~ if I saw the
bug or ~n~ if I didn't.

Here's what the result was:

#+begin_example
You have asked to do an interactive hunt, here's how it goes.
1) I will start a new Emacs instance, which opens a new frame.
2) You will try to reproduce your problem on the new frame.
3) When you’re done, close that frame.
4) I will ask you if you managed to reproduce the problem.
5) We will repeat steps up to 13 times, so hang tight!
Doing some initial tests...
Initial tests done. Hunting for the cause...
"~/sync/emacs/Sacha.el", line 6893 pos 0:
  The assertion returned the following value here:
    t
  Caused by the following expression:
    (use-package highlight-defined :ensure t :custom
      (highlight-defined-face-use-itself t)
      :hook
      (help-mode . highlight-defined-mode)
      (emacs-lisp-mode . highlight-defined-mode))
#+end_example
      
That's weird, since ~highlight-defined~ shouldn't have affected
~mastodon~, but okay. I'm going to skip using ~highlight-defined~ for
now.

If you're tracking down a weird change in your Emacs configuration
file, you might find bug-hunter useful too. It's available on [[https://elpa.gnu.org/packages/bug-hunter.html][ELPA]], or
you can get it from Github.

